# Perl subroutines for reading, manipulating and writing
# Dyana constraint files.
# The location of this file must be specified with a 
# use declaration, such as:
#   use lib "/usr/people/davidg/bin";
# Then, the library must be called with:
#   require "dyana.pl";

# Perl scripts for manipulating DYANA constraint files, Written by
#
#       David P. Goldenberg
#       Department of Biology 
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!

#--------------------------------------------------------------------

sub initialize_upl_arrays {
    @constRes1 = ();
    @constResType1 = ();
    @constAtom1 = ();
    @constRes2 = ();
    @constResType2 = ();
    @constAtom2 = ();
    @constDist = ();
    @constWt = ();
    @keptConst = ();
}

 
sub initialize_aco_arrays {
    @constRes = ();
    @constResType = ();
    @constAngle = ();
    @constLow = ();
    @constUp = ();
    @constWt =();
}



sub write_dyana_dist {
    # this subroutine accepts ten input variables! 
    # The first input variable ($_[0]) is a string identifying a filehandle
    # The second ($_[1]) is a prefix string.
    # The next eight ($_[2] - $_[9] correspond 
    # to the fields of a dyana constraint file record:
    # residue 1, residue type 1, atom 1
    # residue 2, residue type 2, atom 2
    # distance, weight
    # $_[8] is a prefix string
    # $_[9] is a string representing a filehandle
    # It prints to the file identified by the handle the 
    #Dyana distance constraint
    #preceded by the prefix, typically a comment character.
    my ($filehandle);
    $filehandle = $_[0];
    print $filehandle $_[1];
    printf $filehandle "%3s %-4s", $_[2], $_[3];
    printf $filehandle " %-5s ", $_[4];
    printf $filehandle "%3s %-4s", $_[5], $_[6];
    printf $filehandle " %-5s", $_[7];
    printf $filehandle " %6.2f  %8.2e", $_[8], $_[9];
    print $filehandle "\n";
}


sub write_dist_byIndex {
    # this subroutine writes dyana distance constraints
    # identified by an index, using the standard upl arrays
    # initialized in  initialize_upl_arrays.
    # The subroutine accepts three input variables:
    # a string representing a filehandle,
    # a prefix string and an index.
    # It uses write_dyana_dist to write the 
    # dyana constraint with the prefix to the file 
    # identified by the file handle.
    &write_dyana_dist($_[0], $_[1],
		      $constRes1[$_[2]], $constResType1[$_[2]],
		      $constAtom1[$_[2]], 
		      $constRes2[$_[2]], $constResType2[$_[2]],
		      $constAtom2[$_[2]],
                      $constDist[$_[2]], $constWt[$_[2]]);
}


sub write_dyana_aco {
    # this subroutine accepts eight input variables!
    # The first input variable ($_[0]) is a string identifying a filehandle
    # The second ($_[1]) is a prefix string.
    # The next six ($_[2] - $_[7]) correspond 
    # to the fields of a dyana dihedral constraint file record:
    # residue, residue type, angle name
    # dihedral lower limit, dihedral upper limit, weight
    # The subroutine prints to the file identified by the handle the 
    # Dyana dihedral constraint
    # preceded by the prefix, typically a comment character.
    my ($filehandle);
    $filehandle = $_[0];
    print $_[1];
    printf "%4s %-4s", $_[2], $_[3];
    printf "  %-6s", $_[4];
    printf " %6.1f  %6.1f", $_[5], $_[6];
    printf " %8.2e", $_[7];
    print "\n";
}

sub write_aco_byIndex {
    # this subroutine writes dyana dihedral constraints
    # identified by an index, using the standard aco arrays
    # initialized in  initialize_aco_arrays.
    # The subroutine accepts three input variables:
    # a string representing a filehandle,
    # a prefix string, and an index.
    #It uses write_dyana_aco to write the 
    # dyana constraint with the prefix to the file 
    # identified by the file handle.
    &write_dyana_aco($_[0], $_[1],
		      $constRes[$_[2]], $constResType[$_[2]],
		      $constAngle[$_[2]], 
		      $constLow[$_[2]], $constUp[$_[2]],
		      $constWt[$_[2]]);
}


sub read_upl {
    #This subroutine reads dyana distance constraints from 
    # a file identified by an already opened filehandle
    #It accepts two  input variables:
    # a string representing a filehandle and a prefix string
    # Any line containing "#" is written back to standard output
    # If the line does not begin with "#", a "#" is added to the begining.
    # Then, the value of $_[0] is added to the begining
    # The line is then truncated to remove the "#" and everything
    # that follows it.
    # Blank lines are ignored 
    my ($filehandle);
    $filehandle = $_[0];
    while ($constLine = <$filehandle>) {
	if ($constLine =~ /#/) {
	    if ($constLine =~ /^#/) {
		print($_[1],$constLine);
	    } else {
		print($_[1],"#",$constLine);
	    }
	    $hashPos = index($constLine, "#");
	    $constLine = substr($constLine, 0, $hashPos);
	}
	@constField = split (' ',$constLine);
	$noFields = @constField;
	if ($noFields > 0) {
	    #check for lines that begin with residue number
	    if (!($constField[0] =~ /[a-zA-Z]/)) {
	        #print ($constField[0],"\n");
		#check for lines with just residue number & type
		if ($noFields == 2) {
		    @oldRes = ($constField[0], $constField[1]);
		} else {
		    @constRes1 = (@constRes1,$constField[0]);
		    @constResType1 = (@constResType1, $constField[1]);
		    @constAtom1 = (@constAtom1,$constField[2]);
		    @constRes2 = (@constRes2,$constField[3]);
		    @constResType2 = (@constResType2, $constField[4]);
		    @constAtom2 = (@constAtom2,$constField[5]);
		    @constDist = (@constDist, $constField[6]);
		    #check for constraint weight
		    if ($noFields == 8) {
			@constWt = (@constWt, $constField[7]);
		    } else {
			@constWt = (@constWt, 1.0);
		    }
		    @oldRes = ($constField[0], $constField[1]);
		}
	      }
	      #check for lines that begin with atom name
	      if ($constField[0] =~ /[a-zA-Z]/) {
		  #print ($constField[0], "\n");
		  @constRes1 = (@constRes1, $oldRes[0]);
		  @constResType1 = (@constResType1, $oldRes[1]);
		  @constAtom1 = (@constAtom1, $constField[0]);
		  @constRes2 = (@constRes2,$constField[1]);
		  @constResType2 = (@constResType2, $constField[2]);
		  @constAtom2 = (@constAtom2, $constField[3]);
		  @constDist = (@constDist, $constField[4]);
		  #check for constraint weight
		  if ($noFields == 6) {
		      @constWt = (@constWt, $constField[5]);
		  } else {
		      @constWt = (@constWt, 1.0);
		  }
	      }
	}
    }
}


sub read_aco {
    #This subroutine reads dyana dihedral constraints from 
    # an already opened input file identified by a filehandle
    #It accepts two  input variables, a string representing the filehandle
    # and a prefix string, which is added to the begining of all comment lines
    # that are written back to stdout
    # Any line containing "#" is written back to standard output
    # If the line does not begin with "#", a "#" is added to the begining.
    # Then, the value of $_[1] is added to the begining
    # The line is then truncated to remove the "#" and everything
    # that follows it.
    # Blank lines are ignored 
    my ($filehandle);
    $filehandle = $_[0];
    while ($constLine = <$filehandle>) {
	if ($constLine =~ /#/) {
	    if ($constLine =~ /^#/) {
		print($_[1],$constLine);
	    } else {
		print($_[1],"#",$constLine);
	    }
	    $hashPos = index($constLine, "#");
	    $constLine = substr($constLine, 0, $hashPos);

	}
	@constField = split (' ',$constLine);
	$noFields = @constField;
	if ($noFields > 0) {
	    @constRes = (@constRes,$constField[0]);
	    @constResType = (@constResType, $constField[1]);
	    @constAngle = (@constAngle,$constField[2]);
	    @constLow = (@constLow,$constField[3]);
	    @constUp = (@constUp, $constField[4]);
	    #check for constraint weight
	    if ($noFields == 6) {
		@constWt = (@constWt, $constField[5]);
	    } else {
		@constWt = (@constWt, 1.0);
	    }
	 }
    }
}

sub read_bonds {
    #This subroutine reads a list of bonds (eg hydrogen bonds or disulfides) from
    # an input file in dyana-like format
    # The input file is assumed to look just like a dyana distance constraint file,
    # except that there is no distance specified.
    # A constraint weight is optional, as the final field.
    #It accepts two  input variable, a string representing 
    #an already opened filehandle
    # and a prefix string, which is appended to all comment lines
    # that are written back to stdout
    # Any line containing "#" is written back to standard output
    # If the line does not begin with "#", a "#" is added to the begining.
    # Then, the value of $_[1] is added to the begining
    # The line is then truncated to remove the "#" and everything
    # that follows it.
    # Blank lines are ignored 
    my ($filehandle);
    $filehandle = $_[0];
    while ($constLine = <$filehandle>) {
	if ($constLine =~ /#/) {
	    if ($constLine =~ /^#/) {
		print($_[1],$constLine);
	    } else {
		print($_[1],"#",$constLine);
	    }
	    $hashPos = index($constLine, "#");
	    $constLine = substr($constLine, 0, $hashPos);
	}
	@constField = split (' ',$constLine);
	$noFields = @constField;
	if ($noFields > 0) {
	    #check for lines that begin with residue number
	    if (!($constField[0] =~ /[a-zA-Z]/)) {
	        #print ($constField[0],"\n");
		#check for lines with just residue number & type
		if ($noFields == 2) {
		    @oldRes = ($constField[0], $constField[1]);
		} else {
		    @constRes1 = (@constRes1,$constField[0]);
		    @constResType1 = (@constResType1, $constField[1]);
		    @constAtom1 = (@constAtom1,$constField[2]);
		    @constRes2 = (@constRes2,$constField[3]);
		    @constResType2 = (@constResType2, $constField[4]);
		    @constAtom2 = (@constAtom2,$constField[5]);
		    #check for constraint weight
		    if ($noFields == 7) {
			@constWt = (@constWt, $constField[6]);
		    } else {
			@constWt = (@constWt, 1.0);
		    }
		    @oldRes = ($constField[0], $constField[1]);
		}
	      }
	      #check for lines that begin with atom name
	      if ($constField[0] =~ /[a-zA-Z]/) {
		  #print ($constField[0], "\n");
		  @constRes1 = (@constRes1, $oldRes[0]);
		  @constResType1 = (@constResType1, $oldRes[1]);
		  @constAtom1 = (@constAtom1, $constField[0]);
		  @constRes2 = (@constRes2,$constField[1]);
		  @constResType2 = (@constResType2, $constField[2]);
		  @constAtom2 = (@constAtom2, $constField[3]);
		  #check for constraint weight
		  if ($noFields == 5) {
		      @constWt = (@constWt, $constField[6]);
		  } else {
		      @constWt = (@constWt, 1.0);
		  }
	      }
	}
    }
}


sub add_PAcorr {
    # This subroutine adds (or removes) pseudo atom corrections
    # to distance constraints in which an atom is designated by a "Q"
    # It accepts one input variable, an index used to identify 
    # a dyana distance constraint with fields named according
    # to initialize_upl_arrays.
    # It also assumes that the following variables,
    # representing the correction terms, have been initialized:
    # $methyleneCorr
    # $methylCorr
    # $arHdCorr
    # $arHeCorr
    # $isoPropCorr
	if ($constAtom1[$_[0]] eq "QA") {
	    $constDist[$_[0]] += $methyleneCorr;
	} elsif ($constAtom1[$_[0]] eq "QB"){
	    if ($constResType1[$_[0]] eq "ALA") {
		$constDist[$_[0]] += $methylCorr;
	    } else {
		$constDist[$_[0]] += $methyleneCorr;
	    }
	} elsif ($constAtom1[$_[0]] eq "QG") {
	    $constDist[$_[0]] += $methyleneCorr;
	} elsif ($constAtom1[$_[0]] eq "QG1") {
	     if ($constResType1[$_[0]] eq "ILE") {
		 $constDist[$_[0]] += $methyleneCorr;
	     } else {
		 $constDist[$_[0]] += $methylCorr;
	     } 
	} elsif ($constAtom1[$_[0]] eq "QG2") {
	     $constDist[$_[0]] += $methylCorr;
	} elsif ($constAtom1[$_[0]] eq "QQG") {
	    $constDist[$_[0]] += $isoPropCorr;
	} elsif ($constAtom1[$_[0]] eq "QD") {
	    if ($constResType1[$_[0]] eq "TYR" || $constResType1[$_[0]] eq "TYR" ) {	
		$constDist[$_[0]] += $arHdCorr;
	    } else {
		$constDist[$_[0]] += $methyleneCorr;
	    }
	} elsif ($constAtom1[$_[0]] eq "QD1") {
	    $constDist[$_[0]] += $methylCorr;
	} elsif ($constAtom1[$_[0]] eq "QD2") {
	    $constDist[$_[0]] += $methylCorr;
	} elsif ($constAtom1[$_[0]] eq "QQD") {
	    $constDist[$_[0]] += $isoPropCorr;
	} elsif ($constAtom1[$_[0]] eq "QE") {
	    if ($constResType1[$_[0]] eq "TYR" || $constResType1[$_[0]] eq "TYR" ) {
		$constDist[$_[0]] += $arHeCorr;
	    } else {
		$constDist[$_[0]] += $methyleneCorr;
	    }
	}
	if ($constAtom2[$_[0]] eq "QA") {
	    $constDist[$_[0]] += $methyleneCorr;
	} elsif ($constAtom2[$_[0]] eq "QB") {
	    if ($constResType2[$_[0]] eq "ALA") {
		$constDist[$_[0]] += $methylCorr;
	    } else {
		$constDist[$_[0]] += $methyleneCorr;
	    }
	} elsif ($constAtom2[$_[0]] eq "QG") {
	    $constDist[$_[0]] += $methyleneCorr;
	} elsif ($constAtom2[$_[0]] eq "QG1") {
	     if ($constResType2[$_[0]] eq "ILE") {
		 $constDist[$_[0]] += $methyleneCorr;
	     } else {
		 $constDist[$_[0]] += $methylCorr;
	     } 
	} elsif ($constAtom2[$_[0]] eq "QG2") {
	     $constDist[$_[0]] += $methylCorr;
	} elsif ($constAtom2[$_[0]] eq "QQG") {
	    $constDist[$_[0]] += $isoPropCorr;
	} elsif ($constAtom2[$_[0]] eq "QD") {
	    if ($constResType2[$_[0]] eq "TYR" || $constResType2[$_[0]] eq "TYR" ) {	
		$constDist[$_[0]] += $arHdCorr;
	    } else {
		$constDist[$_[0]] += $methyleneCorr;
	    }
	} elsif ($constAtom2[$_[0]] eq "QD1") {
	    $constDist[$_[0]] += $methylCorr;
	} elsif ($constAtom2[$_[0]] eq "QD2") {
	    $constDist[$_[0]] += $methylCorr;
	} elsif ($constAtom2[$_[0]] eq "QQD") {
	    $constDist[$_[0]] += $isoPropCorr;
        } elsif ($constAtom2[$_[0]] eq "QE") {
	    if ($constResType2[$_[0]] eq "TYR" || $constResType2[$_[0]] eq "TYR" ) {
		$constDist[$_[0]] += $arHeCorr;
	    } else {
		$constDist[$_[0]] += $methyleneCorr;
	    }
	}	    
}

#Note: the following line "1" must be present!
1





