# Perl subroutines for manipulating constraints 
# and writing xplor files.
# The location of this file must be specified with a 
# use declaration, such as:
#   use lib "/usr/people/davidg/bin";
# Then, the library must be called with:
#   require "xplor.pl";

# Perl scripts for manipulating DYANA constraint files, Written by
#
#       David P. Goldenberg
#       Department of Biology 
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!

#--------------------------------------------------------------------


sub write_xplor_dist {
    #this subroutine accepts nine input variables:
    # a string identifying a file handle,
    #a prefix value,
    #a first residue number and  atom name (in xplor format)
    #a second residue number and atom name (in xplor format)
    #The three numbers making up an xplor constraint, d, dminus and dplus.
    #It prints the prefix, followed by an 
    #xplor format constraint
    my ($filehandle);
    $filehandle = $_[0];
    printf $filehandle "$_[1]";
    printf $filehandle "assign (resid %3s and name %4s)", $_[2], $_[3];
    printf $filehandle "   (resid %3s and name %4s)", $_[4], $_[5];
    printf $filehandle "  %6.3f   %6.3f   %6.3f  ", $_[6], $_[7], $_[8]; 
    print  "\n";
}

sub calc_xplor_dist {
    #this subroutine accepts one input variable, an upper distance constraint,
    # as from a dyana constraint file.
    #it returns an array of three numbers, @XplorD,corresponding to d, dminus and dplus
    #for the Xplor constraint file
    # dminus is assumed to be 1.8 Angstroms
    # d is calculated as the mean of the upper and lower const
    $upl = $_[0];
    $lol = 1.8;
    $xplorD[0] = $lol + ($upl - $lol)/2; 
    $xplorD[1] = ($upl - $lol)/2;
    $xplorD[2] = $xplorD[1];
    @xplorD;
}





sub write_xplor_aco {
    #this subroutine accepts 14 input variables:
    # a string identifying a file handle, 
    # a prefix string,
    # and the 12 items making up an explor dihedral constraint:
    #  the first residue number and  atom name (in xplor format)
    #  the second residue number and atom name (in xplor format)
    #  the third residue number and  atom name (in xplor format)
    #  the fourth residue number and atom name (in xplor format)
    #  the energy constant
    #  the center of the restraint range, in degrees
    #  the range around the center of the restraint range (1/2 the width of the range)
    #  the exponent of the restraining function.
    #It prints the prefix, followed by an 
    #xplor format constraint
    my ($filehandle);
    $filehandle = $_[0];
    printf $filehandle "$_[1]";
    printf $filehandle "assign (resid %3s and name %4s)",$_[2], $_[3];
    printf $filehandle "    (resid %3s and name %4s)",$_[4] , $_[5];
    print  $filehandle "\n";
    printf $filehandle "       (resid %3s and name %4s)",$_[6] , $_[7];
    printf $filehandle "    (resid %3s and name %4s)",$_[8] , $_[9];
    printf $filehandle "  %6.1f   %6.1f   %6.1f  %2d", $_[10], $_[11], $_[12], $_[13]; 
    print  $filehandle "\n\n";
    
}

sub calc_xplor_aco {
    #this subroutine accepts two input variables, the lower and upper
    #dihedral angle constraints, as from a
    # dyana dihedral constraint file.
    #it returns an array of four numbers, @Xplor,corresponding to :
    # the energy constant - always set to 1
    # the middle of the constrained range
    # the range half-width
    # the exponent of the restraining function - always set to 2
    my ($low, $up, @xplorA);
    $low = $_[0];
    $up = $_[1];
    $xplorA[0] = 1; 
    $xplorA[1] = ($up + $low)/2;
    $xplorA[2] = ($up - $low)/2;
    $xplorA[3] = 2;
    @xplorA;
}

sub dihedral_atoms {
    #this subroutine accepts two input variables:
    #the residue type and the angle name for the dihedral to be specified
    #it returns an eight element array consisting of 
    #the four atom names followed by 
    #four numbers (-1,0,1) used to identify the residue
    #to which the atoms belong
    # it currently recognizes the following dihedral names:
    # PHI, PSI, CHI1, CHI2
    # only upper case letters are recognized in residue names and 
    # dihedral names.
    my ($resType, $angle, @atom, @res);
    $resType = $_[0];
    $angle = $_[1];
    if ($angle =~ /^PHI/i) {
	@res   = (-1, 0, 0, 0);
	@atom = ("C", "N", "CA", "C");
    } elsif ($angle =~ /^PSI/i) {
	@res   = (0, 0, 0, 1);
	@atom = ("N", "CA", "C", "N");
    } elsif ($angle =~ /^CHI1/i) {
	@res = (0, 0, 0, 0);
	@atom[0..2] = ("N", "CA", "CB");
	if ($resType =~ /^SER/i) {
	    $atom[3] = "OG";
	} elsif ($resType =~ /^CYS/i) {
	    $atom[3] = "SG";
	} elsif ($resType =~ /^THR/i) {
	    $atom[3] = "OG1";
	} elsif ($resType =~ /^(VAL|ILE)/i) {
	    $atom[3] = "CG1";
	} else {
	    $atom[3] = "CG";
	}
    } elsif ($angle =~ /^CHI2/i) {
	@res = (0, 0, 0, 0);
	@atom[0..2] = ("CA", "CB", "CG");
	if ($resType =~ /^(ILE|LEU|PHE|TYR|TRP)/i) {
	    $atom[3] = "CD1";
	} elsif ($resType =~ /^HIS/i) {
	    $atom[3] = "ND1";
	} elsif ($resType =~ /^(ASP|ASN)/i) {
	    $atom[3] = "OD1";
	} elsif ($resType =~ /^MET/i) {
	    $atom[3] = "SD";
        } else {
	    $atom[3] = "CD";
	}
    } else {
	print STDERR "Unrecognized dihedral name: $angle";
    }
    (@atom, @res);
}



sub tag_special_atom {
    #This subroutine accepts two input parameters,
    #a residue type and an atom name, both in Dyana format. 
    #The subroutine checks to identify atom designations
    #that are problematic in translating from dyana to xplor format.
    #Those that are problematic are renamed to be treated specially by the 
    #dyana to Xplor dictionary
    my ($resType, $atom);
    $resType = $_[0];
    $atom = $_[1];
    # special case PHE HE1
    if (($resType eq "PHE") && ($atom eq "HE1")) { 
	$atom = "PheHE1";
    }
    # special case HIS HE1
    elsif (($resType eq "HIS") && ($atom eq "HE1")) { 
	$atom = "HisHE1";
    }
    # special case TRP HE
    elsif (($resType eq "TRP") && ($atom eq "HE3")) { 
	$atom = "TrpHE3";
    }
    $atom
}

sub dictionary {
    #this subroutine defines the translations between Dyana
    #and X-plor designations for atoms
    #
#backbone
    $xplor{C} = "C";
    $xplor{O} = "O";
    $xplor{N} = "N";
    $xplor{HN} = "HN";
    $xplor{CA} = "CA";
    $xplor{HA} = "HA";
#Gly alpha protons
    $xplor{HA1} = "HA1";
    $xplor{HA2} = "HA2";
    $xplor{QA}  = "HA#";
#beta methyl, methylene and branched beta
    $xplor{CB} = "CB";
    $xplor{HB1} = "HB3";
    $xplor{HB2} = "HB2";
    $xplor{HB3} = "HB1";
    $xplor{QB} = "HB#";
    $xplor{HB} = "HB";
#gamma methylene and branched gamma
    $xplor{CG} = "CG";
    $xplor{HG2} = "HG2";
    $xplor{HG3} = "HG1";
    $xplor{QG} = "HG#";
    $xplor{GB} = "GB";
#for gamma proton of Ser & Cys
    $xplor{HG} = "HG";
#for gamma proton of Thr;
    $xplor{HG1} = "HG1";
#delta methylene
    $xplor{CD} = "CD";
    $xplor{HD2} = "HD2";
    $xplor{HD3} = "HD1";
    $xplor{QD} = "HD#";
#epsilon methyl and methylene
    $xplor{CE} = "CE";
    $xplor{HE1} = "HE3";
    $xplor{HE2} = "HE2";
    $xplor{HE3} = "HE1";
    $xplor{QE} = "HE#";
#Val and Ile gamma carbons and hydrogens
    $xplor{CG1} = "CG1";
    $xplor{HG11} = "HG13";
    $xplor{HG12} = "HG12";
    $xplor{HG13} = "HG11";
    $xplor{QG1} = "HG1#";
    $xplor{CG2} = "CG2";
    $xplor{HG21} = "HG23";
    $xplor{HG22} = "HG22";
    $xplor{HG23} = "HG21";
    $xplor{QG2} = "HG2#";
    $xplor{QQG} = "HG#";
#Leu and Ile delta carbons and hydrogens
    $xplor{CD1} = "CD1";
    $xplor{HD11} = "HD13";
    $xplor{HD12} = "HD12";
    $xplor{HD13} = "HD11";
    $xplor{QD1} = "HD1#";
    $xplor{CD2} = "CD2";
    $xplor{HD21} = "HD23";
    $xplor{HD22} = "HD22";
    $xplor{HD23} = "HG21";
    $xplor{QD2} = "HD2#";
    $xplor{QQD} = "HD#";
#Phe aromatic ring
    $xplor{HD1} = "HD1";
    $xplor{CE1} = "CE1";
    $xplor{PheHE1} = "HE1"; #this is treated specially, because of epsilon methyl
    $xplor{CE2} = "CE2";
    $xplor{CZ} = "CZ";
    $xplor{HZ} = "HZ";
#Cys gamma sulfur
    $xplor{SG} = "SG";
#Ser hydroxyl
    $xplor{OG} = "OG";
#Thr hydroxyl
    $xplor{OG1} = "OG1";
#Met delta sulfur
    $xplor{SD} = "SD";
#Asp carboxyl and Asn amide
    $xplor{OD1} = "OD1";
    $xplor{OD2} = "OD2";
    $xplor{ND2} = "ND2";
#Glu carboxyl and Gln amide
    $xplor{OE1} = "OE1";
    $xplor{OE2} = "OE2";
    $xplor{NE2} = "NE2";
    $xplor{HE21} = "HE21";
    $xplor{HE22} = "HE22";
    $xplor{QE2} = "HE2#";
#Arg guanido
    $xplor{"NE"} = "NE";
    $xplor{HE} = "HE";
    $xplor{NH1} = "NH1";
    $xplor{NH2} = "NH2";
    $xplor{HH11} = "HH11";
    $xplor{HH12} = "HH12";
    $xplor{QH1} = "HH1#";
    $xplor{HH21} = "HH21";
    $xplor{HH22} = "HH22";
    $xplor{QH2} = "HH2#";
#Lys zeta nitrogen and hydrogens
    $xplor{NZ} = "NZ";
    $xplor{HZ1} = "HZ1";
    $xplor{HZ2} = "HZ2";
    $xplor{HZ3} = "HZ3";
    $xplor{QZ} = "HZ#";
#His imidazole
    $xplor{ND1} = "ND1";
    $xplor{HD1} = "HD1";
    $xplor{HisHE1} = "HE1"; #this is treated specially, because of epsilon methyl
#Trp indole ring
    $xplor{NE1} = "NE1";
    $xplor{CE3} = "CE3";
    $xplor{TrpHE3} = "HE3"; #this is treated specially, because of epsilon methyl
    $xplor{CZ2} = "CZ2";
    $xplor{CZ3} = "CZ3";
    $xplor{HZ2} = "HZ2";
    $xplor{HZ3} = "HZ3";
    $xplor{CH2} = "CH2";
    $xplor{HH2} = "HH2";
#Tyr hydroxyl
    $xplor{OH} = "OH";
    $xplor{HH} = "HH";
}

1
